/******************************************************************************
**
** Source File Name = spclient.c  
**
** Licensed Materials - Property of IBM
**
** (C) COPYRIGHT International Business Machines Corp. 2000 
** All Rights Reserved.
**
** US Government Users Restricted Rights - Use, duplication or
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
**
**
** PURPOSE: This sample program demonstrates stored procedures.
**
**
**    EXTERNAL DEPENDENCIES :
**       - Compile and link loop with the compiler supported 
**         on your platform.
**
** For more information about these samples refer to the README file.
**
** For more information on programming in C, refer to the:
**   -  "Programming in C and C++" section of the Application Development Guide
** For more information on building C applications, refer to the:
**   -  "Building C Applications" section of the Application Building Guide.
**
** For more information on the SQL language refer to the SQL Reference.
**
** spclient contains twelve functions that call stored procedures:
**  (1) callOutLanguage: returns language of the stored procedure library
**        Parameter types used: OUT CHAR(8)
**  (2) callOutParameter: return median salary of EMPLOYEE table
**        Parameter types used: OUT DOUBLE
**                              OUT INTEGER
**                              OUT CHAR(32)
**  (3) callInParameters: accept 3 values and update salaries in EMPLOYEE table
**      accordingly
**        Parameter types used: IN DOUBLE
**                              IN CHAR(3)
**                              OUT INTEGER
**                              OUT CHAR(32)
**  (4) callInoutParameter: accept an input value and return the median
**      salary of the EMPLOYEE table for employees who make more
**      than the input value. Demonstrates how to use null indicators
**      in a client application.
**        Parameter style (C):    GENERAL WITH NULLS
**        Parameter style (Java): JAVA
**        Parameter style (SQL):  SQL
**        Parameter types used: INOUT DOUBLE
**                              OUT INTEGER
**                              OUT CHAR(32)
**  (5) callOneResultSet: return a result set to the client application
**        Parameter types used: IN DOUBLE
**                              OUT INTEGER
**  (6) callTwoResultSets: return two result sets to the client application
**        Parameter types used: IN DOUBLE
**                              OUT INTEGER
**  (7) callClobExtract: return a sub-section of a CLOB data type
**        Parameter types used: IN CHAR(6)
**                              OUT VARCHAR(1000)
**                              OUT INTEGER
**  (8) callDecimalType: pass and receive a DECIMAL data type from a
**      stored procedure
**        Parameter types used: INOUT DECIMAL
**  (9) callAllDataTypes: use all of the common data types in a stored procedure
**        Parameter types used: INOUT SMALLINT
**                              INOUT INTEGER
**                              INOUT BIGINT
**                              INOUT REAL
**                              INOUT DOUBLE
**                              OUT CHAR(1)
**                              OUT CHAR(15)
**                              OUT VARCHAR(12)
**                              OUT DATE
**                              OUT TIME
**  (10) callDB2SQL: calls a DB2SQL parameter style stored procedure
**        Parameter style: DB2SQL
**        Parameter types used: IN CHAR(8)
**                              OUT DOUBLE
**  (11) callDBINFO: calls a stored procedure that receives a DBINFO 
**      structure and returns elements of the structure to the client
**        Parameter types used: IN CHAR(8)
**                              OUT DOUBLE
**                              OUT CHAR(128)
**                              OUT CHAR(8)
**                              OUT INTEGER
**  (12) callProgramTypeMain: calls a stored procedure implemented with
**      PROGRAM TYPE MAIN parameter passing style
**        Parameter types used: IN CHAR(8)
**                              OUT DOUBLE
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sqlcli1.h>
#include <sqlca.h>
#include "utilcli.h"          /* Header file for CLI sample code */


int callOutLanguage(SQLHANDLE, char *);
int callOutParameter(SQLHANDLE, double *);
int callInParameters(SQLHANDLE);
int callInoutParameter(SQLHANDLE, double);
int callClobExtract(SQLHANDLE);
int callOneResultSet(SQLHANDLE, double);
int callTwoResultSets(SQLHANDLE, double);
int callAllDataTypes(SQLHANDLE);
int callDB2SQL(SQLHANDLE, char *);
int callDBINFO(SQLHANDLE);
int callProgramTypeMain(SQLHANDLE);

int main(int argc, char *argv[])
{   SQLRETURN  clirc = SQL_SUCCESS;
    int        rc = 0; 
    SQLHANDLE  henv;  /* environment handle */
    SQLHANDLE  hdbc;  /* connection handles */

    char       dbAlias[SQL_MAX_DSN_LENGTH + 1] ;
    char       user[MAX_UID_LENGTH + 1] ;
    char       pswd[MAX_PWD_LENGTH + 1] ;

    double     medSalary = 0;
    char       language[9];
    
    /* checks the command line arguments */
    rc = CmdLineArgsCheck1(argc, argv, dbAlias, user, pswd);
    if (rc != 0) return(rc) ;

    /* initialize the CLI application */
    rc = CLIAppInit(dbAlias, user, pswd, &henv, &hdbc, 
                     (SQLPOINTER)SQL_AUTOCOMMIT_OFF);
    if (rc != 0) return(rc) ;

    /********************************************************\
    * Call OUT_LANGUAGE stored procedure                     *
    \********************************************************/
    rc = callOutLanguage(hdbc, language);

    /********************************************************\
    * Call OUT_PARAM stored procedure                        *
    \********************************************************/
    rc = callOutParameter(hdbc, &medSalary);

    /********************************************************\
    * Call IN_PARAMS stored procedure                        *
    \********************************************************/
    rc = callInParameters(hdbc);

    /********************************************************\
    * Call INOUT_PARAM stored procedure                      *
    \********************************************************/

    printf("\nCALL stored procedure INOUT_PARAM \n");
    printf("using the median returned by the call to OUT_PARAM\n");
    rc = callInoutParameter(hdbc, medSalary);

    /********************************************************\
    * Call INOUT_PARAM stored procedure again  twice         *
    \********************************************************/
    printf("\nCALL stored procedure named INOUT_PARAM again\n");
    printf("using a NULL input value\n");
    rc = callInoutParameter(hdbc, -99999);

    printf("\nCALL stored procedure INOUT_PARAM again \n");
    printf("Using a value that returns a NOT FOUND error from the "
            "stored procedure\n");
    rc = callInoutParameter(hdbc, 99999.99);
    
    /********************************************************\
    * Call CLOB_EXTRACT stored procedure                     *
    \********************************************************/
    if (strncmp(language, "SQL", 3) == 0) 
    {
      /* SQL procedures cannot handle LOB data types,
         so do not call the CLOB_EXTRACT stored procedure */
        printf("\nStored procedures are implemented in SQL,\n"
               "which cannot handle LOB datatypes\n"
               "Skipping the call to CLOB_EXTRACT\n\n");
    }
    else if (strncmp(language, "C", 1) == 0)
    {
      /* Warn the user that the CLI stored procedure
       * requires a change to the UDF_MEM_SZ variable */
      printf("\n\n   If the CLOB EXTRACT stored procedure is implemented\n"
          "   using CLI, you must increase the value of the UDF_MEM_SZ\n"
          "   database manager configuration variable to at least two\n"
          "   pages larger than the size of the input arguments and\n"
          "   the result of the stored procedure. To do this, issue\n"
          "   the following command from the CLP:\n"
          "      db2 UPDATE DBM CFG USING UDF_MEM_SZ 2048\n"
          "   For the change to take effect, you must then stop and\n"
          "   restart the DB2 server by issuing the following\n"
          "   commands from the CLP:\n"
          "      db2stop\n"
          "      db2start\n");
      rc = callClobExtract(hdbc);
    }	   
    else if (strncmp(language, "JAVA", 4) == 0)
    {
      rc = callClobExtract(hdbc);
    }

    /********************************************************\
    * Call ONE_RESULT_SET stored procedure                   *
    \********************************************************/
    rc = callOneResultSet(hdbc, medSalary);

    /********************************************************\
    * Call TWO_RESULT_SETS stored procedure                  *
    \********************************************************/
    rc = callTwoResultSets(hdbc, medSalary);

    /********************************************************\
    * Call ALL_DATA_TYPES stored procedure                   *
    \********************************************************/
    rc = callAllDataTypes(hdbc);

    if(strncmp(language, "C", 1) != 0)
    {   /* Stored procedures of PARAMETER STYLE DB2SQL, DBINFO,
        or PROGRAM TYPE MAIN can only be implemented by
        LANGUAGE C stored procedures. If language != "C",
        we know that those stored procedures are not
        implemented, and therefore do not call them. */
    }
    else
    {   /********************************************************\
        * Call DB2SQL_EXAMPLE stored procedure                   *
        \********************************************************/
        rc = callDB2SQL(hdbc, "CLERK");
        rc = callDB2SQL(hdbc, "");

        /********************************************************\
        * Call DBINFO_EXAMPLE stored procedure                   *
        \********************************************************/
        rc = callDBINFO(hdbc);

        /********************************************************\
        * Call MAIN_EXAMPLE stored procedure                     *
        \********************************************************/
        rc = callProgramTypeMain(hdbc);
    }

    printf("Roll back the transaction.\n"); 
    clirc = SQLEndTran(SQL_HANDLE_DBC, hdbc, SQL_ROLLBACK);      
    HANDLE_CHECK(SQL_HANDLE_DBC, hdbc, clirc, &henv, &hdbc) ;  
    
    /* terminate the CLI application */
    rc = CLIAppTerm(&henv, &hdbc, dbAlias);
    return(rc) ;
}                                  /* end main */


int callOutLanguage(SQLHANDLE   hdbc,
                 char      * language)
{   SQLRETURN   clirc = SQL_SUCCESS;
    int         rc = 0; 
    SQLHANDLE   hstmt ;  /* statement handle */
    
    char        procName[] = "OUT_LANGUAGE";    
    SQLCHAR   * stmt = (SQLCHAR *) "CALL OUT_LANGUAGE (?)" ;  

    printf("\nCALL stored procedure: %s\n", procName);

    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt) ;
    DBC_HANDLE_CHECK(hdbc, clirc) ;    

    /* prepare the statement */
    clirc = SQLPrepare(hstmt, stmt, SQL_NTS);
    STMT_HANDLE_CHECK(hstmt, clirc);       
    /* bind the parameter to the statement */
    clirc = SQLBindParameter(hstmt, 1, 
      SQL_PARAM_OUTPUT, SQL_C_CHAR, 
      SQL_CHAR, 9, 
      0,language,
      9, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
    
    clirc = SQLExecute(hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);

    printf("Stored procedure returned successfully.\n");

    /********************************************************\
    * Display the language of the stored procedures          *
    \********************************************************/

    printf("Stored procedures are implemented in LANGUAGE: %s\n", language);

    /* free the statement handle */
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt) ;
    STMT_HANDLE_CHECK(hstmt, clirc);
    
    return(rc);
}   /* end callOutLanguage */

int callOutParameter(SQLHANDLE   hdbc, 
                 double      * pOutMedSalary)
{   SQLRETURN   clirc = SQL_SUCCESS;
    int         rc = 0; 
    SQLHANDLE   hstmt ;  /* statement handle */

    SQLINTEGER  outSqlrc;
    char        outMsg[33];
    
    
    char        procName[] = "OUT_PARAM";    
    SQLCHAR   * stmt = (SQLCHAR *) "CALL OUT_PARAM (?,?,?)" ;  

    printf("\nCALL stored procedure: %s\n", procName);

    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt) ;
    DBC_HANDLE_CHECK(hdbc, clirc) ;    

    /* prepare the statement */
    clirc = SQLPrepare(hstmt, stmt, SQL_NTS);
    STMT_HANDLE_CHECK(hstmt, clirc);        

    /* bind the parameters to the statement */
    clirc = SQLBindParameter(hstmt, 1, 
              SQL_PARAM_OUTPUT, SQL_C_DOUBLE, 
              SQL_DOUBLE,0, 
              0, pOutMedSalary,
              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
    
    clirc = SQLBindParameter(hstmt, 2, 
              SQL_PARAM_OUTPUT, SQL_C_LONG, 
              SQL_INTEGER,0, 
              0, &outSqlrc,
              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
    
    clirc = SQLBindParameter(hstmt, 3, 
              SQL_PARAM_OUTPUT, SQL_C_CHAR, 
              SQL_CHAR, 33, 
              0, outMsg,
              33, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);

    clirc = SQLExecute(hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);

    /* Check that the stored procedure executed successfully */
    if (outSqlrc == 0)
    {   printf("Stored procedure returned successfully.\n");

        /***********************************************************\
         * Display the median salary returned as an output parameter *
        \***********************************************************/

        printf("Median salary returned from OUT_PARAM = %8.2f\n", *pOutMedSalary);
    }
    else
    {   /* print the error code and the error message */
        printf("Stored procedure returned SQLCODE %d\n", outSqlrc);
        printf("from procedure section labelled \"%s\".\n", outMsg);
    }

    /* free the statement handle */
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt) ;
    STMT_HANDLE_CHECK(hstmt, clirc);
    
    return (rc);
} /* end callOutParameter */

int   callInParameters(SQLHANDLE hdbc)
{   SQLRETURN   clirc = SQL_SUCCESS;
    int         rc = 0; 
    SQLHANDLE   hstmtSelect, hstmtCall ;  /* statement handle */

    SQLINTEGER  outSqlrc;
    char        outMsg[33];
    
    char        procName[] = "IN_PARAMS";    
    SQLCHAR     stmtSelect[256];    
    SQLCHAR   * stmtCall = (SQLCHAR *) "CALL IN_PARAMS (?,?,?,?,?,?)" ;  

    double      sumOfSalaries;

    /* Declare  variables for passing data to IN_PARAMS */
    double      inLowSal, inMedSal, inHighSal;
    char        inDept[4]; 
    
    /********************************************************\
    * Call IN_PARAMS stored procedure                        *
    \********************************************************/
    inLowSal  = 15000;
    inMedSal  = 20000;
    inHighSal = 25000;
    strcpy(inDept, "E11");
    strcpy(procName, "IN_PARAMS");
    
    printf("\nCALL stored procedure: %s\n", procName);

    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmtSelect) ;
    DBC_HANDLE_CHECK(hdbc, clirc) ;    
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmtCall) ;
    DBC_HANDLE_CHECK(hdbc, clirc) ;    
    
    /* select values from table employee  */

    strcpy((char *)stmtSelect,"SELECT SUM(salary) FROM employee WHERE workdept = '");
    strcat((char *)stmtSelect, inDept);
    strcat((char *)stmtSelect, "'");

    /* bind columns to variables */
    clirc = SQLBindCol(hstmtSelect, 1, SQL_C_DOUBLE, &sumOfSalaries, 0, NULL);
    STMT_HANDLE_CHECK(hstmtSelect, clirc);

    clirc = SQLExecDirect(hstmtSelect, stmtSelect, SQL_NTS) ;
    STMT_HANDLE_CHECK(hstmtSelect, clirc);

    /* fetch the result */
    clirc = SQLFetch(hstmtSelect);
    STMT_HANDLE_CHECK(hstmtSelect, clirc);

    printf("Sum of salaries for dept. %s = %8.2f before calling procedure %s\n",
             inDept, sumOfSalaries, procName);

    /* close the cursor */
    clirc = SQLCloseCursor(hstmtSelect);
    STMT_HANDLE_CHECK(hstmtSelect, clirc);

    clirc = SQLPrepare(hstmtCall, stmtCall, SQL_NTS);
    STMT_HANDLE_CHECK(hstmtCall, clirc);
    
    /* Bind the parameter to the statement */
    clirc = SQLBindParameter(hstmtCall, 1, 
                              SQL_PARAM_INPUT, SQL_C_DOUBLE, 
                              SQL_DOUBLE,0, 
                              0, &inLowSal,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmtCall, clirc);    
    clirc = SQLBindParameter(hstmtCall, 2, 
                              SQL_PARAM_INPUT, SQL_C_DOUBLE, 
                              SQL_DOUBLE,0, 
                              0, &inMedSal,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmtCall, clirc);    
    clirc = SQLBindParameter(hstmtCall, 3, 
                              SQL_PARAM_INPUT, SQL_C_DOUBLE, 
                              SQL_DOUBLE,0, 
                              0, &inHighSal,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmtCall, clirc);    
    clirc = SQLBindParameter(hstmtCall, 4, 
                              SQL_PARAM_INPUT, SQL_C_CHAR, 
                              SQL_CHAR, 4, 
                              0, inDept,
                              4, NULL);
    STMT_HANDLE_CHECK(hstmtCall, clirc);    
    clirc = SQLBindParameter(hstmtCall, 5, 
                              SQL_PARAM_OUTPUT, SQL_C_LONG, 
                              SQL_INTEGER,0, 
                              0, &outSqlrc,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmtCall, clirc);    
    clirc = SQLBindParameter(hstmtCall, 6, 
                              SQL_PARAM_OUTPUT, SQL_C_CHAR, 
                              SQL_CHAR, 33, 
                              0, outMsg,
                              33, NULL);
    STMT_HANDLE_CHECK(hstmtCall, clirc);    

    clirc = SQLExecute(hstmtCall);
    STMT_HANDLE_CHECK(hstmtCall, clirc);    

    /* check that the stored procedure executed successfully */
    if (outSqlrc == 0)
    {   printf("Stored procedure returned successfully.\n");

        /********************************************************\
        * Display the sum salaries for the affected department   *
        \********************************************************/

        /* execute select  */
        clirc = SQLExecDirect(hstmtSelect, stmtSelect, SQL_NTS) ;
        STMT_HANDLE_CHECK(hstmtSelect, clirc);
	    
        /* fetch the sum of salaries */
        clirc = SQLFetch(hstmtSelect);
        STMT_HANDLE_CHECK(hstmtSelect, clirc);
	
        printf("Sum of salaries for dept. %s = %8.2f after calling procedure %s\n",
                 inDept, sumOfSalaries, procName);
    }
    else
    {   /* print the error message and roll back the transaction */
        printf("Stored procedure returned SQLCODE %d\n", outSqlrc);
        printf("from procedure section labelled \"%s\".\n", outMsg);

        printf("Roll back the transaction.\n"); 
        clirc = SQLEndTran(SQL_HANDLE_DBC, hdbc, SQL_ROLLBACK);     
        DBC_HANDLE_CHECK(hdbc, clirc); 
    }

    /* free the statement handles */
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmtSelect) ;
    STMT_HANDLE_CHECK(hstmtSelect, clirc);
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmtCall) ;
    STMT_HANDLE_CHECK(hstmtCall, clirc);

    return (rc);
} /* end callInParameters */

int callInoutParameter(SQLHANDLE hdbc, 
               double    initialMedSalary)
{   SQLRETURN   clirc = SQL_SUCCESS;
    int         rc = 0; 
    SQLHANDLE   hstmt ;  /* statement handle */

    SQLINTEGER  outSqlrc;
    char        outMsg[33];
    SQLINTEGER  medSalaryInd, sqlrcInd, msgInd;
    
    char        procName[] = "INOUT_PARAM";    
    SQLCHAR   * stmt = (SQLCHAR *) "CALL INOUT_PARAM (?,?,?)" ;  

    SQLDOUBLE   inoutMedSalary = 0;  
    
    printf("CALL stored procedure: %s\n", procName);

    /* INOUT_PARAM is PS GENERAL WITH NULLS, so pass null indicators */
    if(initialMedSalary < 0)
    {   /* Salary was negative, indicating a probable error,
           so pass a null value to the stored procedure instead
           by setting medSalaryInd to a negative value */
        medSalaryInd = -1;
        sqlrcInd = -1;
        msgInd = -1;
    }
    else
    {   /* Salary was positive, so pass the value of initialMedSalary
           to the stored procedure by setting medSalaryInd to 0 */
        inoutMedSalary = initialMedSalary;
        medSalaryInd = 0;
        sqlrcInd = 0;
        msgInd = 0;
    }

    
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt) ;
    DBC_HANDLE_CHECK(hdbc, clirc) ;    

    /* prepare the statement */
    clirc = SQLPrepare(hstmt, stmt, SQL_NTS);
    STMT_HANDLE_CHECK(hstmt, clirc);        

    /* bind the parameters to the statement */
    clirc = SQLBindParameter(hstmt, 1, 
                              SQL_PARAM_INPUT_OUTPUT, SQL_C_DOUBLE, 
                              SQL_DOUBLE,0, 
                              0, &inoutMedSalary,
                              0, &medSalaryInd);
    STMT_HANDLE_CHECK(hstmt, clirc);
    
    clirc = SQLBindParameter(hstmt, 2, 
                              SQL_PARAM_OUTPUT, SQL_C_LONG, 
                              SQL_INTEGER,0, 
                              0, &outSqlrc,
                              0, &sqlrcInd);
    STMT_HANDLE_CHECK(hstmt, clirc);
    
    clirc = SQLBindParameter(hstmt, 3, 
                              SQL_PARAM_OUTPUT, SQL_C_CHAR, 
                              SQL_CHAR, 33, 
                              0, outMsg,
                              33, &msgInd);
    STMT_HANDLE_CHECK(hstmt, clirc);

    clirc = SQLExecute(hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);

    
    /* check that the stored procedure executed successfully */
    if(outSqlrc == 0 && medSalaryInd >= 0)
    {   printf("Stored procedure returned successfully.\n");
        printf("Median salary returned from INOUT_PARAM = %8.2f\n",
                inoutMedSalary);
    }
    else if(medSalaryInd < 0)
    {   printf("Stored procedure returned NULL values\n");
    }
    else
    {   /* print the error message */
        printf("Stored procedure returned SQLCODE %d\n", outSqlrc);
        printf("from procedure section labelled \"%s\".\n", outMsg);
    }

    /* free the statement handle */
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt) ;
    STMT_HANDLE_CHECK(hstmt, clirc);
    
    return(rc);
} /* end callInoutParameter */

int callClobExtract(SQLHANDLE hdbc)
{   SQLRETURN  clirc = SQL_SUCCESS;
    int        rc = 0;    
    SQLHANDLE  hstmt;    
    SQLINTEGER outSqlrc;    

    char       procName[] = "CLOB_EXTRACT"; 
    SQLCHAR    stmt[] = "CALL CLOB_EXTRACT (?,?,?)";
    char       inEmpNumber[7];
    char       outDeptInfo[1001];

    printf("\nCALL stored procedure:  %s\n", procName);
    strcpy(inEmpNumber, "000140");

    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt) ;
    DBC_HANDLE_CHECK(hdbc, clirc) ;    

    clirc = SQLPrepare(hstmt, stmt, SQL_NTS);
    STMT_HANDLE_CHECK(hstmt, clirc);

    /* bind paramaters to the statement */
    clirc = SQLBindParameter(hstmt, 1, 
              SQL_PARAM_INPUT, SQL_C_CHAR, 
              SQL_CHAR, 7, 
              0, inEmpNumber,
              7, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
    clirc = SQLBindParameter(hstmt, 2, 
              SQL_PARAM_OUTPUT, SQL_C_CHAR, 
              SQL_VARCHAR,1001, 
              0, outDeptInfo,
              1001, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
    clirc = SQLBindParameter(hstmt, 3, 
             SQL_PARAM_OUTPUT, SQL_C_LONG, 
             SQL_INTEGER,0, 
             0, &outSqlrc,
             0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);

    clirc = SQLExecute(hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);

    /* check that the stored procedure executed successfully */
    if(outSqlrc == 0)
    {   printf("Stored procedure returned successfully.\n");

        /**********************************************************\
        * Display the section of the resume returned from the CLOB *
        \**********************************************************/

        printf("Resume section returned from CLOB_EXTRACT = \n%s\n", 
                outDeptInfo);
    }
    else
    {   /* print the error message */
        printf("Stored procedure returned SQLCODE %d.\n", outSqlrc);
    }

    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);    

    return(rc);
} /* end callClobExtract */

int callOneResultSet(SQLHANDLE hdbc,
                         double    medSalary)
{   SQLRETURN   clirc = SQL_SUCCESS;
    int         rc = 0; 
    SQLHANDLE   hstmt ;  /* statement handle */

    SQLINTEGER  outSqlrc;
    
    char        procName[] = "ONE_RESULT_SET";                          /* :rk.1:erk. */
    SQLCHAR   * stmt = (SQLCHAR *) "CALL ONE_RESULT_SET (?,?)" ;  

    SQLDOUBLE   inSalary, outSalary;
    SQLSMALLINT numCols;
    SQLCHAR     outName[40];
    SQLCHAR     outJob[10];

    inSalary = medSalary;
    

    printf("CALL stored procedure: %s\n", procName);

    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt) ;
    DBC_HANDLE_CHECK(hdbc, clirc) ;    

    /* prepare the statement */
    clirc = SQLPrepare(hstmt, stmt, SQL_NTS);
    STMT_HANDLE_CHECK(hstmt, clirc);        


    /* bind the parameters to the statement */
    clirc = SQLBindParameter(hstmt, 1, 
                              SQL_PARAM_INPUT, SQL_C_DOUBLE, 
                              SQL_DOUBLE,0, 
                              0, &inSalary,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
    clirc = SQLBindParameter(hstmt, 2, 
                              SQL_PARAM_OUTPUT, SQL_C_LONG, 
                              SQL_INTEGER,0, 
                              0, &outSqlrc,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);

    clirc = SQLExecute(hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);

    clirc = SQLNumResultCols(hstmt, &numCols) ;
    STMT_HANDLE_CHECK(hstmt, clirc);

    printf("Result set returned %d columns\n", numCols);

    /* bind columns to variables */
    clirc = SQLBindCol(hstmt, 1, SQL_C_CHAR, outName, 40, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
    clirc = SQLBindCol(hstmt, 2, SQL_C_CHAR, outJob, 10, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
    clirc = SQLBindCol(hstmt, 3, SQL_C_DOUBLE, &outSalary, 0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
  
    /* check that the stored procedure executed successfully */
    if(outSqlrc == 0)
    {   printf("Stored procedure returned successfully.\n");

        /* fetch result set returned from stored procedure */
        clirc = SQLFetch(hstmt);                                       /* :rk.2:erk. */
        STMT_HANDLE_CHECK(hstmt, clirc);

        printf("\nFirst result set returned from %s", procName);
        printf("\n------Name------,  --JOB--, ---Salary--  \n");
        while(clirc != SQL_NO_DATA_FOUND)                              /* :rk.3:erk. */
        {   printf("%15s,%10s,    %.2lf\n", outName, outJob, outSalary);    

            clirc = SQLFetch(hstmt);
            STMT_HANDLE_CHECK(hstmt, clirc);
        }
    }
    else
    {   printf("Stored procedure returned SQLCODE %d\n", outSqlrc);
    }

    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);    

    return(rc);
} /* end callOneResultSet */ 

int callTwoResultSets(SQLHANDLE hdbc,
                          double    medSalary)
{   SQLRETURN   clirc = SQL_SUCCESS;
    int         rc = 0; 
    SQLHANDLE   hstmt ;  /* statement handle */

    SQLINTEGER  outSqlrc;
    
    char        procName[] = "TWO_RESULT_SETS";                          
    SQLCHAR   * stmt = (SQLCHAR *) "CALL TWO_RESULT_SETS (?,?)" ;  

    SQLDOUBLE   inSalary, outSalary;
    SQLSMALLINT numCols;
    SQLCHAR     outName[40];
    SQLCHAR     outJob[10];

    inSalary = medSalary;
    

    printf("\nCALL stored procedure: %s\n", procName);

    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt) ;
    DBC_HANDLE_CHECK(hdbc, clirc) ;    

    /* prepare the statement */
    clirc = SQLPrepare(hstmt, stmt, SQL_NTS);
    STMT_HANDLE_CHECK(hstmt, clirc);        


    /* bind the parameters to the statement */
    clirc = SQLBindParameter(hstmt, 1, 
                              SQL_PARAM_INPUT, SQL_C_DOUBLE, 
                              SQL_DOUBLE,0, 
                              0, &inSalary,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
    clirc = SQLBindParameter(hstmt, 2, 
                              SQL_PARAM_OUTPUT, SQL_C_LONG, 
                              SQL_INTEGER,0, 
                              0, &outSqlrc,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);

    clirc = SQLExecute(hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);

    clirc = SQLNumResultCols(hstmt, &numCols) ;
    STMT_HANDLE_CHECK(hstmt, clirc);

    printf("Result set returned %d columns\n", numCols);

    /* bind columns to variables */
    clirc = SQLBindCol(hstmt, 1, SQL_C_CHAR, outName, 40, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
    clirc = SQLBindCol(hstmt, 2, SQL_C_CHAR, outJob, 10, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
    clirc = SQLBindCol(hstmt, 3, SQL_C_DOUBLE, &outSalary, 0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);

    /* check that the stored procedure executed successfully */
    if(outSqlrc == 0)
    {   printf("Stored procedure returned successfully.\n");
	    
        /* fetch first result set returned from stored procedure */
        clirc = SQLFetch(hstmt);                                    
        STMT_HANDLE_CHECK(hstmt, clirc);

        printf("\nFirst result set returned from %s", procName);
        printf("\n------Name------,  --JOB--, ---Salary--  \n");
        while(clirc != SQL_NO_DATA_FOUND)                           
        {   printf("%15s,%10s,    %.2lf\n", outName, outJob, outSalary);    

            clirc = SQLFetch(hstmt);
            STMT_HANDLE_CHECK(hstmt, clirc);
        }

        clirc = SQLMoreResults(hstmt);
        STMT_HANDLE_CHECK(hstmt, clirc);  	

        /* get next result set, until no more result sets are available */
        while(clirc != SQL_NO_DATA_FOUND)
        {   /* fetch second result set returned from stored procedure */
            clirc = SQLFetch(hstmt); 
            STMT_HANDLE_CHECK(hstmt, clirc); 	    

            printf("\nNext result set returned from %s", procName);
            printf("\n------Name------,  --JOB--, ---Salary--  \n");
            while(clirc != SQL_NO_DATA_FOUND) 
            {   printf("%15s,%10s,    %.2lf\n", outName, outJob, outSalary);    
 
                clirc = SQLFetch(hstmt);
                STMT_HANDLE_CHECK(hstmt, clirc); 		
            }

            clirc = SQLMoreResults(hstmt);
            STMT_HANDLE_CHECK(hstmt, clirc); 	    
        }
	
    }
    else
    {   printf("Stored procedure returned SQLCODE %d\n", outSqlrc);
    }
    
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);    

    return(rc);
} /* end callTwoResultSets */ 

int callAllDataTypes(SQLHANDLE hdbc)
{   SQLRETURN   clirc = SQL_SUCCESS;
    int         rc = 0; 
    SQLHANDLE   hstmt ;  /* statement handle */

    SQLINTEGER  outSqlrc;
    char        outMsg[33];
    
    char        procName[] = "ALL_DATA_TYPES";                          
    SQLCHAR   * stmt = (SQLCHAR *) "CALL ALL_DATA_TYPES (?,?,?,?,?,?,?,?,?,?,?,?)" ;  

    SQLSMALLINT inoutSmallint;
    SQLINTEGER  inoutInteger;
    SQLBIGINT   inoutBigint;
    SQLREAL     inoutReal;
    SQLDOUBLE   inoutDouble;
    SQLCHAR     outChar[2];
    SQLCHAR     outChars[16];
    SQLCHAR     outVarchar[13];
    SQLCHAR     outDate[11];
    SQLCHAR     outTime[9];

    inoutSmallint = 32000;
    inoutInteger  = 2147483000;
    inoutBigint   = 2147480000;
    /* Maximum value of BIGINT is 9223372036854775807 */
    inoutReal     = 100000;
    inoutDouble   = 2500000;

    printf("\nCALL stored procedure: %s\n", procName);

    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt) ;
    DBC_HANDLE_CHECK(hdbc, clirc) ;    

    /* prepare the statement */
    clirc = SQLPrepare(hstmt, stmt, SQL_NTS);
    STMT_HANDLE_CHECK(hstmt, clirc);   	

    /* bind the parameters to the statement */
    clirc = SQLBindParameter(hstmt, 1, 
                              SQL_PARAM_INPUT_OUTPUT, SQL_C_SHORT, 
                              SQL_SMALLINT,0, 
                              0, &inoutSmallint,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);    
    clirc = SQLBindParameter(hstmt, 2, 
                              SQL_PARAM_INPUT_OUTPUT, SQL_C_LONG, 
                              SQL_INTEGER,0, 
                              0, &inoutInteger,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);    
    clirc = SQLBindParameter(hstmt, 3, 
                              SQL_PARAM_INPUT_OUTPUT, SQL_C_SBIGINT,  
                              SQL_BIGINT,0, 
                              0, &inoutBigint,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);    
    clirc = SQLBindParameter(hstmt, 4, 
                              SQL_PARAM_INPUT_OUTPUT, SQL_C_FLOAT, 
                              SQL_REAL,0, 
                              0, &inoutReal,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);    
    clirc = SQLBindParameter(hstmt, 5, 
                              SQL_PARAM_INPUT_OUTPUT, SQL_C_DOUBLE, 
                              SQL_DOUBLE,0, 
                              0, &inoutDouble,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);    
    clirc = SQLBindParameter(hstmt, 6, 
                             SQL_PARAM_OUTPUT, SQL_C_CHAR, 
                             SQL_CHAR,2, 
                             0, outChar,
                             2, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);    
    clirc = SQLBindParameter(hstmt, 7, 
                              SQL_PARAM_OUTPUT, SQL_C_CHAR, 
                              SQL_CHAR, 16, 
                              0, outChars,
                              16, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);    
    clirc = SQLBindParameter(hstmt, 8, 
                              SQL_PARAM_OUTPUT, SQL_C_CHAR, 
                              SQL_VARCHAR, 13, 
                              0, outVarchar,
                              13, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);    
    clirc = SQLBindParameter(hstmt, 9, 
                              SQL_PARAM_OUTPUT, SQL_C_CHAR, 
                              SQL_TYPE_DATE, 11, 
                              0, outDate,
                              11, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);    
    clirc = SQLBindParameter(hstmt, 10, 
                              SQL_PARAM_OUTPUT, SQL_C_CHAR, 
                              SQL_TYPE_TIME, 9, 
                              0, outTime,
                              9, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);    
    clirc = SQLBindParameter(hstmt, 11, 
                              SQL_PARAM_OUTPUT, SQL_C_LONG, 
                              SQL_INTEGER,0, 
                              0, &outSqlrc,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);    
    clirc = SQLBindParameter(hstmt, 12, 
                              SQL_PARAM_OUTPUT, SQL_C_CHAR, 
                              SQL_CHAR, 33, 
                              0, outMsg,
                              33, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);    

    clirc = SQLExecute(hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);    

    /* check that the stored procedure executed successfully */
    if(outSqlrc == 0)
    {   printf("Stored procedure returned successfully.\n");

        printf("Value of SMALLINT = %d\n", inoutSmallint);
        printf("Value of INTEGER = %ld\n", inoutInteger);
        printf("Value of BIGINT = %lld\n", inoutBigint);
        printf("Value of REAL = %.2f\n", inoutReal);
        printf("Value of DOUBLE = %.2lf\n", inoutDouble);
        printf("Value of CHAR(1) = %s\n", outChar);
        printf("Value of CHAR(15) = %s\n", outChars);
        printf("Value of VARCHAR(12) = %s\n", outVarchar);
        printf("Value of DATE = %s\n", outDate);
        printf("Value of TIME = %s\n", outTime);
    }
    else
    {   /* print the error message */
        printf("Stored procedure returned SQLCODE %d\n", outSqlrc);
        printf("from procedure section labelled \"%s\".\n", (char *) outMsg);
    }

    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);    
 
    return(rc);
} /* end callAllDataTypes */

int callDB2SQL(SQLHANDLE hdbc, char * jobName)
{   SQLRETURN   clirc = SQL_SUCCESS;
    int         rc = 0; 
    SQLHANDLE   hstmt ;  /* statement handle */

    SQLINTEGER  salaryInd, jobInd;
    
    char        procName[] = "DB2SQL_EXAMPLE";                          
    SQLCHAR   * stmt = (SQLCHAR *) "CALL DB2SQL_EXAMPLE (?, ?)" ; 

    SQLCHAR     inJob[9];
    SQLDOUBLE   outSalary = 0;

    SQLCHAR     message[200] ;
    SQLCHAR     sqlstate[7] ;
    SQLINTEGER  sqlcode ;
    SQLSMALLINT length ;
    

    strcpy((char *)inJob, jobName);
    printf("\nCALL stored procedure: %s\n", procName);
    /* DB2SQL_EXAMPLE is PS DB2SQL, so pass
       a null indicator for each parameter */
    salaryInd = 0;

    /* inJob is an IN parameter, so check to see
       if there is any input value */
    if(strlen((char*)inJob) == 0)
    {   /* inJob is null, so set the null indicator
	   to -1 to indicate there is no input value */
        jobInd = -1;
        printf("with NULL input, to return a custom SQLSTATE");
        printf(" and diagnostic message\n");
    }
    else
    {   /* inJob is not null, so set the null indicator
	   to 0 to indicate there is an input value */
        jobInd = strlen((char *)inJob);
    }

    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt) ;
    DBC_HANDLE_CHECK(hdbc, clirc) ;    
	
    /* prepare the statement */
    clirc = SQLPrepare(hstmt, stmt, SQL_NTS);
    STMT_HANDLE_CHECK(hstmt, clirc);   	
    

    /* bind the parameters to the statement */
    clirc = SQLBindParameter(hstmt, 1,
                              SQL_PARAM_INPUT, SQL_C_CHAR,
                              SQL_CHAR, 9,
                              0, inJob,
                              9, &jobInd);
    STMT_HANDLE_CHECK(hstmt, clirc);
  
    clirc = SQLBindParameter(hstmt, 2, 
                              SQL_PARAM_OUTPUT, SQL_C_DOUBLE, 
                              SQL_DOUBLE,0, 
                              0, &outSalary,
                              0, &salaryInd);
    STMT_HANDLE_CHECK(hstmt, clirc);

    clirc = SQLExecute(hstmt);

    SQLGetDiagRec(SQL_HANDLE_STMT,
                   hstmt,
                   1,
                   sqlstate,
                   &sqlcode,
                   message,
                   200,
                   &length) ;   

    /* check that the stored procedure executed successfully */
    if(strncmp((char *)sqlstate, "00000", 5) == 0) 
    {   printf("Stored procedure returned successfully.\n");
        printf("Average salary for job %s = %9.2lf\n", inJob, outSalary);
    }
    else
    {   printf("%s", message);
    }

    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);  
    
    return(rc);
}

int callDBINFO(SQLHANDLE hdbc)
{   SQLRETURN   clirc = SQL_SUCCESS;
    int         rc = 0; 
    SQLHANDLE   hstmt ;  /* statement handle */

    SQLINTEGER  outSqlrc;
    
    char        procName[] = "DBINFO_EXAMPLE";                          
    SQLCHAR   * stmt = (SQLCHAR *) "CALL DBINFO_EXAMPLE (?,?,?,?,?)" ; 

    SQLCHAR     inJob[9];
    SQLDOUBLE   outSalary;
    SQLCHAR     outDbname[129];  /* name of database from DBINFO structure */
    SQLCHAR     outDbVersion[9]; /* version of database from DBINFO structure */


    strcpy((char *)inJob, "MANAGER");
    
    printf("\nCALL stored procedure: %s\n", procName);

    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt) ;
    DBC_HANDLE_CHECK(hdbc, clirc) ;    

    /* prepare the statement */
    clirc = SQLPrepare(hstmt, stmt, SQL_NTS);
    STMT_HANDLE_CHECK(hstmt, clirc);   	
	

    /* bind the parameters to the statement */
    clirc = SQLBindParameter(hstmt, 1,
                              SQL_PARAM_INPUT, SQL_C_CHAR,
                              SQL_CHAR, 9,
                              0, inJob,
                              9, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);

    clirc = SQLBindParameter(hstmt, 2, 
                              SQL_PARAM_OUTPUT, SQL_C_DOUBLE, 
                              SQL_DOUBLE,0, 
                              0, &outSalary,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
    clirc = SQLBindParameter(hstmt, 3,
                              SQL_PARAM_OUTPUT, SQL_C_CHAR,
                              SQL_CHAR, 129,
                              0, outDbname,
                              129, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
    clirc = SQLBindParameter(hstmt, 4,
                              SQL_PARAM_OUTPUT, SQL_C_CHAR,
                              SQL_CHAR, 9,
                              0, (char *) outDbVersion,
                              9, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
    clirc = SQLBindParameter(hstmt, 5, 
                              SQL_PARAM_OUTPUT, SQL_C_LONG, 
                              SQL_INTEGER,0, 
                              0, &outSqlrc,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
  
    clirc = SQLExecute(hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);

    /* check that the stored procedure executed successfully */
    if(outSqlrc == 0) 
    {   printf("Stored procedure returned successfully.\n");
        printf("Average salary for job %s = %9.2lf\n", inJob, outSalary);
        printf("Database name from DBINFO structure = %s\n", outDbname);
        printf("Database version from DBINFO structure = %s\n", outDbVersion);
    }
    else
    {   printf("Stored procedure returned SQLCODE %d\n", outSqlrc);
    }

    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);  
    
    return(rc);
} /* end callDBINFO */


int callProgramTypeMain(SQLHANDLE hdbc)
{   SQLRETURN   clirc = SQL_SUCCESS;
    int         rc = 0; 
    SQLHANDLE   hstmt ;  /* statement handle */

    SQLINTEGER  outSqlrc;
    
    char        procName[] = "MAIN_EXAMPLE";                          
    SQLCHAR   * stmt = (SQLCHAR *) "CALL MAIN_EXAMPLE (?,?,?)" ; 

    SQLCHAR     inJob[9];
    SQLDOUBLE   outSalary;
	
    strcpy((char *)inJob, "DESIGNER");
    printf("\nCALL stored procedure: %s\n", procName);

    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt) ;
    DBC_HANDLE_CHECK(hdbc, clirc) ;    

    /* prepare the statement */
    clirc = SQLPrepare(hstmt, stmt, SQL_NTS);
    STMT_HANDLE_CHECK(hstmt, clirc);   	
    

    /* bind the parameters to the statement */
    clirc = SQLBindParameter(hstmt, 1,
                              SQL_PARAM_INPUT, SQL_C_CHAR,
                              SQL_CHAR, 9,
                              0, inJob,
                              9, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);
  
    clirc = SQLBindParameter(hstmt, 2, 
                              SQL_PARAM_OUTPUT, SQL_C_DOUBLE, 
                              SQL_DOUBLE,0, 
                              0, &outSalary,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);

    clirc = SQLBindParameter(hstmt, 3, 
                              SQL_PARAM_OUTPUT, SQL_C_LONG, 
                              SQL_INTEGER,0, 
                              0, &outSqlrc,
                              0, NULL);
    STMT_HANDLE_CHECK(hstmt, clirc);

    clirc = SQLExecute(hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);

    /* check that the stored procedure executed successfully */
    if(outSqlrc == 0) 
    {   printf("Stored procedure returned successfully.\n");
        printf("Average salary for job %s = %9.2lf\n", inJob, outSalary);
    }
    else
    {   printf("Stored procedure returned SQLCODE %d\n", outSqlrc);
    }

    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    STMT_HANDLE_CHECK(hstmt, clirc);  

    return(rc);
} /* end callProgramTypeMain */

