/*******************************************************************************
**
** Source File Name = spserver.c  
**
** Licensed Materials - Property of IBM
**
** (C) COPYRIGHT International Business Machines Corp. 2000
** All Rights Reserved.
**
** US Government Users Restricted Rights - Use, duplication or
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
**
**
** PURPOSE : This program demonstrates stored procedures implemented in CLI.
** 
** There are two parts to this program:
**           - the spclient executable (placed on the client)
**           - the spserver library (placed on the server)
**
** BUILDING THE SHARED LIBRARY:
** 1. Ensure the Database Manager Configuration file has the keyword KEEPDARI
** set to "no". This allows unloading shared libraries while developing stored
** procedures. You can view the file's settings by entering the command:
** "db2 GET DBM CFG". You can set KEEPDARI to "no" with this command:
** "db2 UPDATE DBM CFG USING KEEPDARI no".
** NOTE: Setting KEEPDARI to "no" causes a performance cost when accessing the stored 
** procedures because they have to be reloaded into memory each time they are called.
** If this is a consideration, use the KEEPDARI "yes" setting, and stop and restart
** DB2 before building the shared library, by entering "db2stop" followed by "db2start". 
** This forces DB2 to unload shared libraries and enables the build file or the makefile 
** to delete a previous version of the shared library from the "function" directory. 
** 2. To build the shared library, enter "bldclisp spserver" (UNIX & OS/2), "make 
** spserver" (UNIX), "nmake spserver" (OS/2 & Windows), or for the Microsoft Visual
** C++ compiler on Windows, enter "bldmclis spserver", or for the VisualAge C++ compiler
** on Windows, enter "bldvclis spserver".
** 3. To catalog the stored procedures in the shared library, first connect to the 
** database by entering: "db2 connect to sample".
** 4. If the stored procedures were previously cataloged for a different language, 
** uncatalog them by entering: "db2 -td@ -vf spdrop.db2".
** 5. Then, catalog the stored procedures by entering: "db2 -td@ -vf spcreate.db2".
** 6. Before calling the CLOB_EXTRACT stored procedure, you must increase
** the value of your UDF_MEM_SZ database manager configuration variable
** to at least two pages larger than the size of the input arguments and
** the result of the stored procedure.
**
** To do this, issue the following command from the CLP:
**   "db2 UPDATE DBM CFG USING UDF_MEM_SZ 2048"
**
** For the change to take effect, you must then stop and start the
** DB2 server by issuing the following commands from the CLP:
**   "db2stop"
**   "db2start"
**
**
** For more information about these samples see the README file.
**
** For more information on programming in CLI see the:
**     - "Building CLI Applications" section of the Application Building Guide, and the
**     - CLI Guide and Reference.
**
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sqlcli1.h>
#include <sqlca.h>
#include <sqludf.h>
#include "utilcli.h"


/* Macros for Handle Checking */
#define SRV_HANDLE_CHECK(htype, hndl, CLIrc, henv, hdbc)            \
    if (CLIrc == SQL_INVALID_HANDLE)                                \
    {   StpCleanUp(henv, hdbc);                                      \
        return 0;                                                     \
    }	                                                              \
    if (CLIrc == SQL_ERROR)                                         \
    {   StpCleanUp(henv, hdbc);                                      \
        return 0;                                                     \
    }	                                                              

#define SRV_HANDLE_CHECK_SETTING_SQLRC(htype, hndl, CLIrc, henv, hdbc, pSqlrc)    \
    if (CLIrc == SQL_INVALID_HANDLE)                                              \
    {   *pSqlrc = 0;                                                                \
        StpCleanUp(henv, hdbc);                                                    \
        return 0;                                                                   \
    }	                                                                            \
    if (CLIrc == SQL_ERROR)                                                       \
    {   SqlrcSet(htype, hndl, henv, hdbc, pSqlrc);                                 \
        StpCleanUp(henv, hdbc);                                                    \
        return 0;                                                                   \
    }	                                                              

#define SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(htype, hndl, CLIrc, henv, hdbc,   \
                                                pSqlrc, outMsg, inMsg)            \
    if (CLIrc == SQL_INVALID_HANDLE)                                            \
    {   *pSqlrc =  0;                                                             \
        strcpy(outMsg, inMsg);                                                   \
        StpCleanUp(henv, hdbc);                                                  \
        return 0;                                                                 \
    }	                                                                          \
    if (CLIrc == SQL_ERROR)                                                     \
    {   SqlrcAndMsgSet(htype, hndl, henv, hdbc, pSqlrc, outMsg, inMsg);          \
        StpCleanUp(henv, hdbc);                                                  \
        return 0;                                                                 \
    }	 



    
void StpCleanUp(SQLHANDLE henv, SQLHANDLE hdbc)
{   SQLDisconnect(hdbc);
    SQLFreeHandle(SQL_HANDLE_DBC, hdbc);	
    SQLFreeHandle(SQL_HANDLE_ENV, henv);    
}

void SqlrcSet(SQLSMALLINT htype,
               SQLHANDLE   hndl,
               SQLHANDLE   henv,
               SQLHANDLE   hdbc, 
               sqlint32  * pSqlrc)
{   struct sqlca sqlca;

    /* initialize sqlcode */	
    sqlca.sqlcode = 0;
	
    switch(htype)
    {   case SQL_HANDLE_ENV:
            SQLGetSQLCA(hndl, SQL_NULL_HDBC, SQL_NULL_HSTMT, &sqlca);
            break;
        case SQL_HANDLE_DBC:
            SQLGetSQLCA(henv, hndl, SQL_NULL_HSTMT, &sqlca);
            break;
        case SQL_HANDLE_STMT:
            SQLGetSQLCA(henv, hdbc, hndl, &sqlca);	    
            break;
        default:
            break;	    
    }	    
    *pSqlrc = sqlca.sqlcode;    
}	

void SqlrcAndMsgSet(SQLSMALLINT htype,
                     SQLHANDLE   hndl,
                     SQLHANDLE   henv,
                     SQLHANDLE   hdbc, 
                     sqlint32  * pSqlrc,
                     char      * outMsg,
                     char      * inMsg)
{   strcpy(outMsg, inMsg);
    SqlrcSet(htype, hndl, henv, hdbc, pSqlrc);	
}	

/* Declare function prototypes for this stored procedure library */

SQL_API_RC SQL_API_FN outlanguage(char *);
SQL_API_RC SQL_API_FN out_param(double *, sqlint32 *, char *);
SQL_API_RC SQL_API_FN in_params(double *, double *, double *, char *, sqlint32 *, char *);
SQL_API_RC SQL_API_FN inout_param(double *inoutMedian, sqlint32 *, char *, sqlint16 *);
SQL_API_RC SQL_API_FN one_result_set_to_caller(double *, sqlint32 *);
SQL_API_RC SQL_API_FN two_result_sets(double *, sqlint32 *);
SQL_API_RC SQL_API_FN extract_from_clob(char *, char *, sqlint32 *);
SQL_API_RC SQL_API_FN all_data_types(sqlint16 *, sqlint32 *, sqlint64 *,float *, double *, char *,
          char *, char *, char *, char *, sqlint32 *, char *);
SQL_API_RC SQL_API_FN db2sql_example(char *, double *, sqlint16 *, char *, char *, char *, char *);
SQL_API_RC SQL_API_FN dbinfo_example(char *, double *, char *, char *, sqlint32 *, struct sqludf_dbinfo *);
SQL_API_RC SQL_API_FN main_example(int, char **);

SQL_API_RC SQL_API_FN 
outlanguage(char outLanguage[9]) 
{   SQLHANDLE  henv, hdbc=0, hstmt;
    SQLRETURN  clirc;
    SQLCHAR    stmt[100];
    
    /* Initialize strings used for output parameters to NULL */
    memset(outLanguage, '\0', 9);

    /*-----------------------------------------------------------------*/
    /* Setup CLI required environment                                  */
    /*-----------------------------------------------------------------*/
    clirc = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
    SRV_HANDLE_CHECK(SQL_HANDLE_ENV, henv, clirc, henv, hdbc);
    
    clirc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
    SRV_HANDLE_CHECK(SQL_HANDLE_ENV, henv, clirc, henv, hdbc);

    /* set AUTOCOMMIT off */
    clirc = SQLSetConnectAttr(hdbc,
                            SQL_ATTR_AUTOCOMMIT,
                            SQL_AUTOCOMMIT_OFF, SQL_NTS);
    SRV_HANDLE_CHECK(SQL_HANDLE_DBC, hdbc, clirc, henv, hdbc);

    /*-----------------------------------------------------------------*/
    /* Issue NULL Connect, since in CLI we need a statement handle     */
    /* and thus a connection handle and environment handle.            */
    /* A connection is not established, rather the current             */
    /* connection from the calling application is used                 */
    /*-----------------------------------------------------------------*/

    clirc = SQLConnect(hdbc, NULL, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS);
    SRV_HANDLE_CHECK(SQL_HANDLE_DBC, hdbc, clirc, henv, hdbc);
    
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    SRV_HANDLE_CHECK(SQL_HANDLE_DBC, hdbc, clirc, henv, hdbc);

    strcpy((char*)stmt,"SELECT LANGUAGE FROM sysibm.sysprocedures ");
    strcat((char*)stmt,"WHERE procname = 'OUT_LANGUAGE' ");

    clirc = SQLExecDirect(hstmt, stmt, SQL_NTS);
    SRV_HANDLE_CHECK(SQL_HANDLE_STMT, hstmt, clirc, henv, hdbc);


    /* bind columns to variables */
    clirc = SQLBindCol(hstmt, 1, SQL_C_CHAR, outLanguage, 9, NULL);
    SRV_HANDLE_CHECK(SQL_HANDLE_STMT, hstmt, clirc, henv, hdbc);

    /* fetch each row */
    clirc = SQLFetch(hstmt);
    SRV_HANDLE_CHECK(SQL_HANDLE_STMT, hstmt, clirc, henv, hdbc);

    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    SRV_HANDLE_CHECK(SQL_HANDLE_STMT, hstmt, clirc, henv, hdbc);
    
    clirc = SQLDisconnect(hdbc);
    SRV_HANDLE_CHECK(SQL_HANDLE_DBC, hdbc, clirc, henv, hdbc);

    clirc = SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
    SRV_HANDLE_CHECK(SQL_HANDLE_DBC, hdbc, clirc, henv, hdbc);

    clirc = SQLFreeHandle(SQL_HANDLE_ENV,  henv);
    SRV_HANDLE_CHECK(SQL_HANDLE_ENV, henv, clirc, henv, hdbc);
    return(0);
} /* end outlanguage function */

SQL_API_RC SQL_API_FN 
out_param(double   * pOutMedianSalary,
           sqlint32 * pOutSqlrc, 
           char       outMsg[33])
{   SQLHANDLE     henv, hdbc=0, hstmt1, hstmt2;
    SQLRETURN     clirc;
    SQLCHAR       stmt1[100], stmt2[100];

    sqlint16      numRecords;
    double        medianSalary;
    int           counter;

    counter = 0;
    *pOutSqlrc = 0;

    /* Initialize strings used for output parameters to NULL */
    memset(outMsg, '\0', 33);

    /*-----------------------------------------------------------------*/
    /* Setup CLI required environment                                  */
    /*-----------------------------------------------------------------*/
    clirc = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");
    
    clirc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");

    /* set AUTOCOMMIT off */
    clirc = SQLSetConnectAttr(hdbc,
                            SQL_ATTR_AUTOCOMMIT,
                            SQL_AUTOCOMMIT_OFF, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLSetConnectAttr");

    /*-----------------------------------------------------------------*/
    /* Issue NULL Connect, since in CLI we need a statement handle     */
    /* and thus a connection handle and environment handle.            */
    /* A connection is not established, rather the current             */
    /* connection from the calling application is used                 */
    /*-----------------------------------------------------------------*/

    clirc = SQLConnect(hdbc, NULL, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLConnect");
    
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt2);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");
    
    strcpy((char*)stmt1, "SELECT count(*)  FROM staff ");
    clirc = SQLExecDirect(hstmt1, stmt1, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLExecDirect");

    strcpy((char*)stmt2, "SELECT CAST(salary AS DOUBLE)  FROM staff ");
    strcat((char*)stmt2, "order by salary");
    clirc = SQLExecDirect(hstmt2, stmt2, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLExecDirect");

    /* bind columns to variables */
    clirc = SQLBindCol(hstmt1, 1, SQL_C_SHORT, &numRecords , 0, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLBindCol");

    clirc = SQLBindCol(hstmt2, 1, SQL_C_DOUBLE, &medianSalary , 0, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLBindCol");

    /* fetch number of rows in "staff" table */
    clirc = SQLFetch(hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFetch");
    
    for(counter=0; counter < (numRecords/2+1); counter++)
    {   
        clirc = SQLFetch(hstmt2);
        SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2,
                clirc, henv, hdbc, pOutSqlrc, outMsg, "SQLFetch");
    }	    

    /* Set value of OUT parameter to host variable */
    *pOutMedianSalary = medianSalary;

    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt2);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");    

    clirc = SQLDisconnect(hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLDisconnect");

    clirc = SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");

    clirc = SQLFreeHandle(SQL_HANDLE_ENV,  henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");
    return(0);

} /* end out_param function */


SQL_API_RC SQL_API_FN
in_params(double   * pInLowSal, 
          double   * pInMedSal,
          double   * pInHighSal,
          char       inDepartment[4],
          sqlint32 * pOutSqlrc, 
          char       outMsg[33])
{   SQLHANDLE    henv, hdbc=0, hstmt1, hstmt2;
    SQLRETURN    clirc;
    SQLCHAR      stmt1[100], stmt2[100];

    double       lowSal;
    double       medSal;
    double       highSal;
    double       salary;
    double       tmp;
    SQLCHAR      cursorName[20];
    SQLSMALLINT  cursorNameLen;

    *pOutSqlrc = 0;

    /* Initialize strings used for output parameters to NULL */
    memset(outMsg, '\0', 33);
    
    lowSal  = *pInLowSal;
    medSal  = *pInMedSal;
    highSal = *pInHighSal;

    /*-----------------------------------------------------------------*/
    /* Setup CLI required environment                                  */
    /*-----------------------------------------------------------------*/
    clirc = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");
    
    clirc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");

    /* set AUTOCOMMIT off */
    clirc = SQLSetConnectAttr(hdbc,
                            SQL_ATTR_AUTOCOMMIT,
                            SQL_AUTOCOMMIT_OFF, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLSetConnectAttr");

    /*-----------------------------------------------------------------*/
    /* Issue NULL Connect, since in CLI we need a statement handle     */
    /* and thus a connection handle and environment handle.            */
    /* A connection is not established, rather the current             */
    /* connection from the calling application is used                 */
    /*-----------------------------------------------------------------*/

    clirc = SQLConnect(hdbc, NULL, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLConnect");
    
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt2);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");

    strcpy((char*)stmt1,"SELECT CAST(salary AS DOUBLE) FROM employee ");
    strcat((char*)stmt1,"WHERE workdept = ? FOR UPDATE OF salary");

    clirc = SQLPrepare(hstmt1, stmt1, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLPrepare");

    /* Bind the parameter to the statement */
    clirc = SQLBindParameter(hstmt1, 1,
                         SQL_PARAM_INPUT, SQL_C_CHAR,
                         SQL_CHAR, 4,
                         0, inDepartment,
                         4, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLBindParameter");
    
    clirc = SQLExecute(hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLExecute");

    /* bind columns to variables */
    clirc = SQLBindCol(hstmt1, 1, SQL_C_DOUBLE, &salary , 0, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLBindCol");

    /* get Cursor of the SELECT statement's handle */
    clirc = SQLGetCursorName(hstmt1, cursorName, 20, &cursorNameLen);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLGetCursorName");

    sprintf((char *)stmt2,
            "UPDATE employee SET salary = ?  WHERE CURRENT OF %s ",
            cursorName);

    clirc = SQLPrepare(hstmt2, stmt2, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLPrepare");

    /* fetch first row */
    clirc = SQLFetch(hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFetch");
    if (clirc == SQL_NO_DATA_FOUND)      
    {   *pOutSqlrc =  100;                  
        strcpy(outMsg, "No Data Found");      

        /* Free resources before returning */
        SQLFreeHandle(SQL_HANDLE_STMT, hstmt1);
        SQLFreeHandle(SQL_HANDLE_STMT, hstmt2);
        StpCleanUp(henv, hdbc);     
        return 0;                    
    }	
    
    while(clirc != SQL_NO_DATA_FOUND)
    {   if (salary < lowSal) 
        {   clirc = SQLBindParameter(hstmt2, 1,
                                  SQL_PARAM_INPUT, SQL_C_DOUBLE,
                                  SQL_DOUBLE,0,
                                  0, &lowSal,
                                  0, NULL);
            SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2,
                  clirc, henv, hdbc, pOutSqlrc, outMsg, "SQLBindParameter");
	
            clirc = SQLExecute(hstmt2);
            SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2,
                  clirc, henv, hdbc, pOutSqlrc, outMsg, "SQLExecute");
        }
        else if (salary < medSal)
        {   clirc = SQLBindParameter(hstmt2, 1,
                                     SQL_PARAM_INPUT, SQL_C_DOUBLE,
                                     SQL_DOUBLE,0,
                                     0, &medSal,
                                     0, NULL);
            SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2,
                  clirc, henv, hdbc, pOutSqlrc, outMsg, "SQLBindParameter");	 
            clirc = SQLExecute(hstmt2);
            SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2,
                  clirc, henv, hdbc, pOutSqlrc, outMsg, "SQLExecute");
        }
        else if (salary < highSal)
        {   clirc = SQLBindParameter(hstmt2, 1,
                                     SQL_PARAM_INPUT, SQL_C_DOUBLE,
                                     SQL_DOUBLE,0,
                                     0, &highSal,
                                     0, NULL);
            SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2,
                  clirc, henv, hdbc, pOutSqlrc, outMsg, "SQLBindParameter");	 
            clirc = SQLExecute(hstmt2);
            SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2,
                  clirc, henv, hdbc, pOutSqlrc, outMsg, "SQLExecute");
        }
        else
        {   tmp=salary*1.1;
            clirc = SQLBindParameter(hstmt2, 1,
                                     SQL_PARAM_INPUT, SQL_C_DOUBLE,
                                     SQL_DOUBLE,0,
                                     0, &tmp,
                                     0, NULL);
            SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2,
                  clirc, henv, hdbc, pOutSqlrc, outMsg, "SQLBindParameter");	 
            clirc = SQLExecute(hstmt2);
            SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2,
                  clirc, henv, hdbc, pOutSqlrc, outMsg, "SQLExecute");
        }

        clirc = SQLFetch(hstmt1);
        SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                      henv, hdbc, pOutSqlrc, outMsg, "SQLFetch");
    }

    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt2);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");    

    clirc = SQLDisconnect(hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLDisconnect");

    clirc = SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");

    clirc = SQLFreeHandle(SQL_HANDLE_ENV,  henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");
    
    return(0);
} /* end in_params function */

SQL_API_RC SQL_API_FN 
inout_param(double   * pInoutMedian,
            sqlint32 * pOutSqlrc,
            char       outMsg[33],
            sqlint16   inoutNullinds[3])
{   SQLHANDLE    henv, hdbc=0, hstmt1, hstmt2;
    SQLRETURN    clirc;
    SQLCHAR      stmt1[100], stmt2[100];
	
    sqlint16     numRecords;
    double       medianSalary;
    double       tmpSalary;
    int          counter;

    counter = 0; 
    numRecords = 0;

    /* Initialize strings used for output parameters to NULL */
    memset(outMsg, '\0', 33);

    if (inoutNullinds[0] < 0) {
      /* NULL value was received as input, so return NULL output */
      /* The index of the null indicator array corresponds to the
       * explicit parameter of the stored procedure */
      inoutNullinds[0] = -1; 
      inoutNullinds[1] = -1;
      inoutNullinds[2] = -1;
      return 0;
    }
    else 
    {
      /* The stored procedure will return values,
       * so set the null indicators accordingly */
      inoutNullinds[1] = 0;
      inoutNullinds[2] = 0;
    }

    /* Initialize the output parameters */
    *pOutSqlrc = 0;
    
    medianSalary = *pInoutMedian;
    /*-----------------------------------------------------------------*/
    /* Setup CLI required environment                                  */
    /*-----------------------------------------------------------------*/
    clirc = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");
    
    clirc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");

    /* set AUTOCOMMIT off */
    clirc = SQLSetConnectAttr(hdbc,
                            SQL_ATTR_AUTOCOMMIT,
                            SQL_AUTOCOMMIT_OFF, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLSetConnectAttr");

    /*-----------------------------------------------------------------*/
    /* Issue NULL Connect, since in CLI we need a statement handle     */
    /* and thus a connection handle and environment handle.            */
    /* A connection is not established, rather the current             */
    /* connection from the calling application is used                 */
    /*-----------------------------------------------------------------*/

    clirc = SQLConnect(hdbc, NULL, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLConnect");
    
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt2);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");

    strcpy((char*)stmt1,"SELECT count(*)  FROM staff ");
    strcat((char*)stmt1,"WHERE salary > ? ");

    strcpy((char*)stmt2,"SELECT CAST(salary AS DOUBLE)  FROM staff ");
    strcat((char*)stmt2,"where salary > ? order by salary ");

    clirc = SQLPrepare(hstmt1, stmt1, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLPrepare");
    clirc = SQLPrepare(hstmt2, stmt2, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLPrepare");

    clirc = SQLBindParameter(hstmt1, 1,
                         SQL_PARAM_INPUT, SQL_C_DOUBLE,
                         SQL_DOUBLE, 0,
                         0, &medianSalary,
                         0, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLBindParameter");
    /* Bind the parameter to the statement */
    clirc = SQLBindParameter(hstmt2, 1,
                         SQL_PARAM_INPUT, SQL_C_DOUBLE,
                         SQL_DOUBLE, 0,
                         0, &medianSalary,
                         0, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLBindParameter");

    clirc = SQLBindCol(hstmt1, 1, SQL_C_SHORT, &numRecords, 0,NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLBindCol");

    /* bind columns to variables */
    clirc = SQLBindCol(hstmt2, 1, SQL_C_DOUBLE, &tmpSalary , 0, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLBindCol");

    clirc = SQLExecute(hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLExecute");

    clirc = SQLFetch(hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1,
              clirc, henv, hdbc, pOutSqlrc, outMsg, "SQLFetch");
 
    if (numRecords == 0)      
    {   *pOutSqlrc =  100; 
        strcpy(outMsg, "No Data Found");	    

        /* Free resources before returning */
        SQLFreeHandle(SQL_HANDLE_STMT, hstmt2);
        SQLFreeHandle(SQL_HANDLE_STMT, hstmt1);
        StpCleanUp(henv, hdbc);     
        return 0;                    
    }	  

    clirc = SQLExecute(hstmt2);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLExecute");
    
    for(counter=0; counter < (numRecords/2+1); counter++)
    {   
        clirc = SQLFetch(hstmt2);
        SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2,
                  clirc, henv, hdbc, pOutSqlrc, outMsg, "SQLFetch");
    }

    /* Set value of INOUT parameter to median salary */
    *pInoutMedian = tmpSalary;

    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt2);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");    

    clirc = SQLDisconnect(hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLDisconnect");

    clirc = SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");

    clirc = SQLFreeHandle(SQL_HANDLE_ENV,  henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");
    return(0);
    
} /* end inout_param function */


SQL_API_RC SQL_API_FN
one_result_set_to_caller(double   * pInSalary,
                          sqlint32 * pOutSqlrc)
{   SQLHANDLE  henv, hdbc=0, hstmt;
    SQLRETURN  clirc;
    SQLCHAR    stmt[100];

    *pOutSqlrc = 0;

    /*-----------------------------------------------------------------*/
    /* Setup CLI required environment                                  */
    /*-----------------------------------------------------------------*/
    clirc = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc);
    
    clirc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc);

    /* set AUTOCOMMIT off */
    clirc = SQLSetConnectAttr(hdbc,
                            SQL_ATTR_AUTOCOMMIT,
                            SQL_AUTOCOMMIT_OFF, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    /*-----------------------------------------------------------------*/
    /* Issue NULL Connect, since in CLI we need a statement handle     */
    /* and thus a connection handle and environment handle.            */
    /* A connection is not established, rather the current             */
    /* connection from the calling application is used                 */
    /*-----------------------------------------------------------------*/

    clirc = SQLConnect(hdbc, NULL, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);
    
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);
    
    strcpy((char*)stmt,"SELECT name, job, CAST(salary AS DOUBLE) AS salary");
    strcat((char*)stmt," FROM staff WHERE salary > ? ORDER BY salary");
    
    clirc = SQLPrepare(hstmt, stmt, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt, clirc,
                  henv, hdbc, pOutSqlrc);

    /* Bind the parameter to the statement */
    clirc = SQLBindParameter(hstmt, 1,
                         SQL_PARAM_INPUT, SQL_C_DOUBLE,
                         SQL_DOUBLE,0,
                         0, pInSalary,
                         0, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLExecute(hstmt);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLDisconnect(hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLFreeHandle(SQL_HANDLE_ENV,  henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc);
    return(0);

} /* end one_result_set_to_caller function */

SQL_API_RC SQL_API_FN
two_result_sets(double   * pInSalary,
                 sqlint32 * pOutSqlrc)
{   SQLHANDLE  henv, hdbc=0, hstmt1, hstmt2;
    SQLRETURN  clirc;
    SQLCHAR    stmt1[100], stmt2[100];

    *pOutSqlrc = 0;

    /*-----------------------------------------------------------------*/
    /* Setup CLI required environment                                  */
    /*-----------------------------------------------------------------*/
    clirc = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc);
    
    clirc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc);

    /* set AUTOCOMMIT off */
    clirc = SQLSetConnectAttr(hdbc,
                            SQL_ATTR_AUTOCOMMIT,
                            SQL_AUTOCOMMIT_OFF, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    /*-----------------------------------------------------------------*/
    /* Issue NULL Connect, since in CLI we need a statement handle     */
    /* and thus a connection handle and environment handle.            */
    /* A connection is not established, rather the current             */
    /* connection from the calling application is used                 */
    /*-----------------------------------------------------------------*/

    clirc = SQLConnect(hdbc, NULL, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);
    
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt2);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);
    
    /* Statement for first result set:
       staff with a salary greater than the IN parameter */
    strcpy((char*)stmt1,"SELECT name, job, CAST(salary AS DOUBLE) AS salary");
    strcat((char*)stmt1," FROM staff WHERE salary > ? ORDER BY salary");

    /* Statement for second result set:
       staff with a salary less than the IN parameter */
    strcpy((char*)stmt2,"SELECT name, job, CAST(salary AS DOUBLE) AS salary");
    strcat((char*)stmt2," FROM staff WHERE salary < ? ");
    strcat((char*)stmt2,"ORDER BY salary DESC");

    clirc = SQLPrepare(hstmt1, stmt1, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLPrepare(hstmt2, stmt2, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc);

    /* Bind the parameter to the statement */
    clirc = SQLBindParameter(hstmt1, 1,
                         SQL_PARAM_INPUT, SQL_C_DOUBLE,
                         SQL_DOUBLE,0,
                         0, pInSalary,
                         0, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc);
    clirc = SQLBindParameter(hstmt2, 1,
                         SQL_PARAM_INPUT, SQL_C_DOUBLE,
                         SQL_DOUBLE,0,
                         0, pInSalary,
                         0, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLExecute(hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc);
    clirc = SQLExecute(hstmt2);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLDisconnect(hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLFreeHandle(SQL_HANDLE_ENV,  henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc);

    return(0);
} /* end two_result_sets function */

/* Before calling the CLOB_EXTRACT CLI stored procedure, you must increase
 * the value of your UDF_MEM_SZ database manager configuration variable
 * to at least two pages larger than the size of the input arguments and
 * the result of the stored procedure.
 *
 * To do this, issue the following command from the CLP:
 *   db2 UPDATE DBM CFG USING UDF_MEM_SZ 2048
 *
 * For the change to take effect, you must then stop and start the
 * DB2 server by issuing the following commands from the CLP:
 *   db2stop
 *   db2start
 */

SQL_API_RC SQL_API_FN
extract_from_clob(char                 inEmpNumber[7],           
                  char                 outDeptInfo[1001], 
                  sqlint32           * pOutSqlrc)
{   SQLHANDLE    henv, hdbc=0;
    SQLRETURN    clirc;

    SQLHANDLE    hstmtClobSelect, hstmtLocUse, hstmtLocFree ; 	
    SQLCHAR    * stmtClobSelect = 
                      (SQLCHAR *) "SELECT resume FROM emp_resume "
                      "WHERE empno = ?  AND resume_format = 'ascii'" ;
    SQLCHAR    * stmtLocFree = (SQLCHAR *) "FREE LOCATOR ?" ;    

    SQLINTEGER   clobLoc ;   /* A LOB locator used for resume */
    SQLINTEGER   pcbValue ;

    SQLINTEGER   clobPieceLen, clobLen ;
    SQLUINTEGER  clobPieceBegin, clobPieceEnd ;


    *pOutSqlrc = 0;

    /* Initialize strings used for output parameters to NULL */
    memset(outDeptInfo, '\0', 1001);

    /*-----------------------------------------------------------------*/
    /* Setup CLI required environment                                  */
    /*-----------------------------------------------------------------*/
    clirc = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc);
    
    clirc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc);

    /* set AUTOCOMMIT off */
    clirc = SQLSetConnectAttr(hdbc,
                            SQL_ATTR_AUTOCOMMIT,
                            SQL_AUTOCOMMIT_OFF, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    /*-----------------------------------------------------------------*/
    /* Issue NULL Connect, since in CLI we need a statement handle     */
    /* and thus a connection handle and environment handle.            */
    /* A connection is not established, rather the current             */
    /* connection from the calling application is used                 */
    /*-----------------------------------------------------------------*/

    clirc = SQLConnect(hdbc, NULL, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

/* --------------- fetch CLOB data --------------------------------------*/
    
    /* allocate a statement handle */
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmtClobSelect) ;
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    /* Bind the parameter to the statement */
    clirc = SQLBindParameter(hstmtClobSelect, 1,
                         SQL_PARAM_INPUT, SQL_C_CHAR,
                         SQL_VARCHAR, 7,
                         0, inEmpNumber,
                         7, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmtClobSelect, clirc,
                  henv, hdbc, pOutSqlrc);
    
    /* execute directly the statement */ 
    clirc = SQLExecDirect(hstmtClobSelect, stmtClobSelect, SQL_NTS) ;
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmtClobSelect, clirc,
                  henv, hdbc, pOutSqlrc); 
    
    /* bind CLOB column to LOB locator */
    clirc = SQLBindCol(hstmtClobSelect, 1, SQL_C_CLOB_LOCATOR,
                        &clobLoc, 0, &pcbValue);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmtClobSelect, clirc,
                  henv, hdbc, pOutSqlrc);
    
    /* fetch the CLOB data */ 
    clirc = SQLFetch(hstmtClobSelect);
    if (clirc == SQL_NO_DATA_FOUND)      
    {   *pOutSqlrc =  100;                  
        StpCleanUp(henv, hdbc);     
        return 0;                    
    }	    

/* ---------------- work with the LOB locator -----------------------------*/

    /* allocate a statement handle */
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmtLocUse) ;
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);
    
    /* get the length of the whole CLOB data */
    clirc = SQLGetLength(hstmtLocUse, SQL_C_CLOB_LOCATOR,
                          clobLoc, &clobLen, NULL) ;
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmtLocUse, clirc,
                  henv, hdbc, pOutSqlrc);

    /* get the starting postion of the CLOB piece of data */
    clirc = SQLGetPosition(hstmtLocUse,
                            SQL_C_CLOB_LOCATOR,
                            clobLoc,
                            0,
                            (SQLCHAR *) "Department Information",
                            strlen("Department Information"),
                            1,
                            &clobPieceBegin,
                            NULL) ;
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmtLocUse, clirc,
                  henv, hdbc, pOutSqlrc);    
    /* get the ending postion of the CLOB piece of data */
    clirc = SQLGetPosition(hstmtLocUse,
                            SQL_C_CLOB_LOCATOR,
                            clobLoc,
                            0,
                            (SQLCHAR *) "Education",
                            strlen("Education"),
                            1,
                            &clobPieceEnd,
                            NULL) ;
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmtLocUse, clirc,
                  henv, hdbc, pOutSqlrc);

    /* read the piece of CLOB data in outDeptInfo */
    clirc = SQLGetSubString(hstmtLocUse,
                             SQL_C_CLOB_LOCATOR,
                             clobLoc,
                             clobPieceBegin,
                             clobPieceEnd - clobPieceBegin,
                             SQL_C_CHAR,
                             outDeptInfo,
                             clobPieceEnd - clobPieceBegin + 1,
                             &clobPieceLen,
                             NULL) ;
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmtLocUse, clirc,
                  henv, hdbc, pOutSqlrc);    
    
/* ---------------- free the LOB locator ----------------------------------*/    

    /* allocate a statement handle */
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmtLocFree) ;
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLSetParam(hstmtLocFree,
                         1,
                         SQL_C_CLOB_LOCATOR,
                         SQL_CLOB_LOCATOR,
                         0,
                         0,
                         &clobLoc,
                         NULL) ;
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmtLocFree, clirc,
                  henv, hdbc, pOutSqlrc);
    
    /* execute directly the statement */ 
    clirc = SQLExecDirect(hstmtLocFree, stmtLocFree, SQL_NTS) ;
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmtLocFree, clirc,
                  henv, hdbc, pOutSqlrc);

/* ------------------ terminate the stored procedure -----------------------*/
    
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmtClobSelect);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmtClobSelect, clirc,
                  henv, hdbc, pOutSqlrc);
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmtLocUse);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmtLocUse, clirc,
                  henv, hdbc, pOutSqlrc);
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmtLocFree);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmtLocFree, clirc,
                  henv, hdbc, pOutSqlrc);
    
    clirc = SQLDisconnect(hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLFreeHandle(SQL_HANDLE_ENV,  henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc);

    return(0);
} /* end extract_from_clob procedure */

SQL_API_RC SQL_API_FN
all_data_types(sqlint16 * pInoutSmall,  sqlint32 * pInoutInt,
               sqlint64 * pInoutBig,    float   * pInoutReal, 
               double   * pInoutDouble, char    outChar[2],
               char       outChars[16], char    outVarchar[13],
               char       outDate[11],  char    outTime[9],
               sqlint32 * pOutSqlrc,    char    outMsg[33])
{   SQLHANDLE   henv, hdbc=0;
    SQLHANDLE   hstmt1, hstmt2, hstmt3, hstmt4, hstmt5;
    SQLCHAR     stmt1[100], stmt2[100], stmt3[100], stmt4[100], stmt5[100];
    SQLRETURN   clirc;

    char        firstName[13];        /* VARCHAR(12) */

    *pOutSqlrc = 0;

    /* Initialize strings used for output parameters to NULL */
    memset(outChar, '\0', 2);
    memset(outChars, '\0', 16);
    memset(outVarchar, '\0', 13);
    memset(outMsg, '\0', 33);
    
    if (*pInoutSmall == 0) { *pInoutSmall = 1; }
    else { *pInoutSmall = (*pInoutSmall / 2); }

    if (*pInoutInt == 0) { *pInoutInt = 1; }
    else { *pInoutInt = (*pInoutInt / 2); }
 
    if (*pInoutBig == 0) { *pInoutBig = 1; }
    else { *pInoutBig = (*pInoutBig / 2); }

    if (*pInoutReal == 0) { *pInoutReal = 1; }
    else { *pInoutReal = (*pInoutReal / 2); }

    if (*pInoutDouble == 0) { *pInoutDouble = 1; }
    else { *pInoutDouble = (*pInoutDouble / 2); }

    /*-----------------------------------------------------------------*/
    /* Setup CLI required environment                                  */
    /*-----------------------------------------------------------------*/
    clirc = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");
    
    clirc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");

    /* set AUTOCOMMIT off */
    clirc = SQLSetConnectAttr(hdbc,
                            SQL_ATTR_AUTOCOMMIT,
                            SQL_AUTOCOMMIT_OFF, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLSetConnectAttr");

    /*-----------------------------------------------------------------*/
    /* Issue NULL Connect, since in CLI we need a statement handle     */
    /* and thus a connection handle and environment handle.            */
    /* A connection is not established, rather the current             */
    /* connection from the calling application is used                 */
    /*-----------------------------------------------------------------*/

    clirc = SQLConnect(hdbc, NULL, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLConnect");
    
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt2);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt3);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt4);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt5);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLAllocHandle");

    strcpy((char*)stmt1,"SELECT midinit from employee  ");
    strcat((char*)stmt1,"WHERE empno = '000180'");
    
    strcpy((char*)stmt2,"SELECT lastname from employee  ");
    strcat((char*)stmt2,"WHERE empno = '000180'");
    
    strcpy((char*)stmt3,"SELECT firstnme from employee  ");
    strcat((char*)stmt3,"WHERE empno = '000180'");
    
    strcpy((char*)stmt4,"values CURRENT DATE  ");

    strcpy((char*)stmt5,"values CURRENT TIME  ");

    /* statement 1 */    
    clirc = SQLExecDirect(hstmt1, stmt1, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLExecDirect");
    clirc = SQLBindCol(hstmt1, 1, SQL_C_CHAR, outChar , 2, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLBindCol");
    clirc = SQLFetch(hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFetch");
    
    /* statement 2 */
    clirc = SQLExecDirect(hstmt2, stmt2, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLExecDirect");
    clirc = SQLBindCol(hstmt2, 1, SQL_C_CHAR, outChars , 16, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLBindCol");
    clirc = SQLFetch(hstmt2);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFetch");

    /* statement 3 */    
    clirc = SQLExecDirect(hstmt3, stmt3, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt3, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLExecDirect");
    clirc = SQLBindCol(hstmt3, 1, SQL_C_CHAR, outVarchar, 13, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt3, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLBindCol");
    clirc = SQLFetch(hstmt3);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt3, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFetch");
    
    /* statement 4 */
    clirc = SQLExecDirect(hstmt4, stmt4, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt4, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLExecDirect");    
    clirc = SQLBindCol(hstmt4, 1, SQL_C_CHAR, outDate , 11, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt4, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLBindCol");
    clirc = SQLFetch(hstmt4);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt4, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFetch");

    /* statement 5 */
    clirc = SQLExecDirect(hstmt5, stmt5, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt5, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLExecDirect");
    clirc = SQLBindCol(hstmt5, 1, SQL_C_CHAR, outTime , 9, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt5, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLBindCol");
    clirc = SQLFetch(hstmt5);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt5, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFetch");

    /* end the stored procedure */
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt1);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt1, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt2);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt2, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");    
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt3);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt3, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt4);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt4, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt5);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_STMT, hstmt5, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");    
    
    clirc = SQLDisconnect(hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLDisconnect");

    clirc = SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");

    clirc = SQLFreeHandle(SQL_HANDLE_ENV,  henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC_AND_MSG(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc, outMsg, "SQLFreeHandle");

    return(0);
} /* end all_data_types procedure */

SQL_API_RC SQL_API_FN 
dbinfo_example(char                   inJob[9],    
                double               * pOutSalary,    
                char                   outDbName[129], 
                char                   outDbVersion[9],
                sqlint32             * pOutSqlrc,
                struct sqludf_dbinfo * pOutDbInfo)
{   SQLHANDLE    henv, hdbc=0, hstmt;
    SQLRETURN    clirc;
    SQLCHAR      stmt[100];

    /* Initialize strings used for output parameters to NULL */
    memset(outDbName, '\0', 129);
    memset(outDbVersion, '\0', 9);

    *pOutSqlrc = 0;

    /*-----------------------------------------------------------------*/
    /* Setup CLI required environment                                  */
    /*-----------------------------------------------------------------*/
    clirc = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc);
    
    clirc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc);

    /* set AUTOCOMMIT off */
    clirc = SQLSetConnectAttr(hdbc,
                            SQL_ATTR_AUTOCOMMIT,
                            SQL_AUTOCOMMIT_OFF, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    /*-----------------------------------------------------------------*/
    /* Issue NULL Connect, since in CLI we need a statement handle     */
    /* and thus a connection handle and environment handle.            */
    /* A connection is not established, rather the current             */
    /* connection from the calling application is used                 */
    /*-----------------------------------------------------------------*/

    clirc = SQLConnect(hdbc, NULL, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);
    
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    strcpy((char*)stmt,"SELECT AVG(salary) FROM employee WHERE job = ?");

    clirc = SQLPrepare(hstmt, stmt, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt, clirc,
                  henv, hdbc, pOutSqlrc);
    clirc = SQLBindParameter(hstmt, 1,
                         SQL_PARAM_INPUT, SQL_C_CHAR,
                         SQL_CHAR, 9,
                         0, inJob,
                         9, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt, clirc,
                  henv, hdbc, pOutSqlrc);
    clirc = SQLExecute(hstmt);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt, clirc,
                  henv, hdbc, pOutSqlrc);    
    clirc = SQLBindCol(hstmt, 1, SQL_C_DOUBLE, pOutSalary , 0, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt, clirc,
                  henv, hdbc, pOutSqlrc);    
    clirc = SQLFetch(hstmt);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt, clirc,
                  henv, hdbc, pOutSqlrc);    
    
    /* Copy values from the DBINFO structure into the output parameters.
     * You must explicitly null-terminate the strings.
     */
    strncpy (outDbName, (char *)(pOutDbInfo->dbname), pOutDbInfo->dbnamelen);
    outDbName[pOutDbInfo->dbnamelen] = '\0';
    strncpy (outDbVersion, (char *)(pOutDbInfo->ver_rel), 8);
    outDbVersion[8] = '\0';

    /* terminate the stored procedure */    
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt, clirc,
                  henv, hdbc, pOutSqlrc);
    
    clirc = SQLDisconnect(hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLFreeHandle(SQL_HANDLE_ENV,  henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc);

    return(0);
} /* end dbinfo_example function */

SQL_API_RC SQL_API_FN
main_example(int     argc, 
              char ** argv)
{   SQLHANDLE    henv, hdbc=0, hstmt;
    SQLRETURN    clirc;
    SQLCHAR      stmt[100];

    char         inJob[9];    
    double       * pOutSalary = NULL;     
    sqlint32     * pOutSqlrc;    

    /* argv[0] contains the procedure name, so parameters start at argv[1] */
    strcpy(inJob, (char *) argv[1]); 
    pOutSalary = (double *) argv[2];     
    pOutSqlrc  = (sqlint32 *) argv[3];

    *pOutSqlrc = 0;
        
    /*-----------------------------------------------------------------*/
    /* Setup CLI required environment                                  */
    /*-----------------------------------------------------------------*/
    clirc = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc);
    
    clirc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc);

    /* set AUTOCOMMIT off */
    clirc = SQLSetConnectAttr(hdbc,
                            SQL_ATTR_AUTOCOMMIT,
                            SQL_AUTOCOMMIT_OFF, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    /*-----------------------------------------------------------------*/
    /* Issue NULL Connect, since in CLI we need a statement handle     */
    /* and thus a connection handle and environment handle.            */
    /* A connection is not established, rather the current             */
    /* connection from the calling application is used                 */
    /*-----------------------------------------------------------------*/

    clirc = SQLConnect(hdbc, NULL, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);
    
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);
    
    strcpy((char*)stmt, "SELECT AVG(salary) FROM employee WHERE job = ?");

    clirc = SQLPrepare(hstmt, stmt, SQL_NTS);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt, clirc,
                  henv, hdbc, pOutSqlrc);

    /* Bind the parameter to the statement */
    clirc = SQLBindParameter(hstmt, 1,
                         SQL_PARAM_INPUT, SQL_C_CHAR,
                         SQL_CHAR, 9,
                         0, inJob,
                         9, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLExecute(hstmt);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt, clirc,
                  henv, hdbc, pOutSqlrc);
     
    /* bind columns to variables */
    clirc = SQLBindCol(hstmt, 1, SQL_C_DOUBLE, pOutSalary , 0, NULL);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt, clirc,
                  henv, hdbc, pOutSqlrc);
 
    clirc = SQLFetch(hstmt);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt, clirc,
                  henv, hdbc, pOutSqlrc);

    /* terminate the stored procedure */    
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_STMT, hstmt, clirc,
                  henv, hdbc, pOutSqlrc);
    
    clirc = SQLDisconnect(hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_DBC, hdbc, clirc,
                  henv, hdbc, pOutSqlrc);

    clirc = SQLFreeHandle(SQL_HANDLE_ENV,  henv);
    SRV_HANDLE_CHECK_SETTING_SQLRC(SQL_HANDLE_ENV, henv, clirc,
                  henv, hdbc, pOutSqlrc);
    
    return(0);
} /* end main_example function */


SQL_API_RC SQL_API_FN 
db2sql_example(char        inJob[9],          
                double    * pOutSalary,     
                sqlint16    inoutNullinds[2], 
                char        outSqlState[6],    
                char        inQualName[28],    
                char        inSpecName[19],    
                char        outDiagMsg[71]) 
{   SQLHANDLE    henv, hdbc=0, hstmt;
    SQLRETURN    clirc;
    SQLCHAR      stmt[100];


    if (inoutNullinds[0] < 0)
    {   /* NULL value was received as input, so return NULL output */
        /* Using the RETURNS NULL ON NULL CALL clause in your
        CREATE PROCEDURE statement is more efficient */
        inoutNullinds[1] = -1;

        /* Set custom SQLSTATE to return to client. */
        strcpy(outSqlState, "38100");

        /* Set custom message to return to client. Note that although the
        OUT parameter is declared as CHAR(70), DB2 prepends the
        procedure name and shared library entry point to the message.
        Keep the custom message short to avoid truncation. */
        strcpy(outDiagMsg, "Received null input.");

        return(0);
    }
    else 
    {
      /* The stored procedure will return a value for the second
       * explicit parameter, so set the null indicator accordingly */
      inoutNullinds[1] = 0;
    }

    *pOutSalary   = 0;

    /*-----------------------------------------------------------------*/
    /* Setup CLI required environment                                  */
    /*-----------------------------------------------------------------*/
    clirc = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
    SRV_HANDLE_CHECK(SQL_HANDLE_ENV, henv, clirc, henv, hdbc);

    clirc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
    SRV_HANDLE_CHECK(SQL_HANDLE_ENV, henv, clirc, henv, hdbc);

    /* set AUTOCOMMIT off */
    clirc = SQLSetConnectAttr(hdbc,
                            SQL_ATTR_AUTOCOMMIT,
                            SQL_AUTOCOMMIT_OFF, SQL_NTS);
    SRV_HANDLE_CHECK(SQL_HANDLE_DBC, hdbc, clirc, henv, hdbc);

    /*-----------------------------------------------------------------*/
    /* Issue NULL Connect, since in CLI we need a statement handle     */
    /* and thus a connection handle and environment handle.            */
    /* A connection is not established, rather the current             */
    /* connection from the calling application is used                 */
    /*-----------------------------------------------------------------*/

    clirc = SQLConnect(hdbc, NULL, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS);
    SRV_HANDLE_CHECK(SQL_HANDLE_DBC, hdbc, clirc, henv, hdbc);
    
    clirc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    SRV_HANDLE_CHECK(SQL_HANDLE_DBC, hdbc, clirc, henv, hdbc);
    
    strcpy((char*)stmt,"SELECT (CAST(AVG(salary) AS DOUBLE)) FROM employee "); 
    strcat((char*)stmt,"WHERE job = ?"); 

    clirc = SQLPrepare(hstmt, stmt, SQL_NTS);
    SRV_HANDLE_CHECK(SQL_HANDLE_STMT, hstmt, clirc, henv, hdbc);

    /* Bind the parameter to the statement */
    clirc = SQLBindParameter(hstmt, 1,
                         SQL_PARAM_INPUT, SQL_C_CHAR,
                         SQL_CHAR, 9,
                         0, inJob,
                         9, NULL);
    SRV_HANDLE_CHECK(SQL_HANDLE_STMT, hstmt, clirc, henv, hdbc);
    
    clirc = SQLExecute(hstmt);
    SRV_HANDLE_CHECK(SQL_HANDLE_STMT, hstmt, clirc, henv, hdbc);
    
   /* bind columns to variables */
    clirc = SQLBindCol(hstmt, 1, SQL_C_DOUBLE, pOutSalary , 0, NULL);
    SRV_HANDLE_CHECK(SQL_HANDLE_STMT, hstmt, clirc, henv, hdbc);

    clirc = SQLFetch(hstmt);
    SRV_HANDLE_CHECK(SQL_HANDLE_STMT, hstmt, clirc, henv, hdbc);

    /* terminate the stored procedure */    
    clirc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    SRV_HANDLE_CHECK(SQL_HANDLE_STMT, hstmt, clirc, henv, hdbc);
    
    clirc = SQLDisconnect(hdbc);
    SRV_HANDLE_CHECK(SQL_HANDLE_DBC, hdbc, clirc, henv, hdbc);

    clirc = SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
    SRV_HANDLE_CHECK(SQL_HANDLE_DBC, hdbc, clirc, henv, hdbc);

    clirc = SQLFreeHandle(SQL_HANDLE_ENV,  henv);
    SRV_HANDLE_CHECK(SQL_HANDLE_ENV, henv, clirc, henv, hdbc);
    
    return(0);
} /* end db2sql_example function */
  

